#include "blc_program.h"
#include "graph.h"
#include <pthread.h>
#include <sys/time.h>
#include <unistd.h>

using namespace std;

void create_graph(FILE *pipef, deque <blc_channel>&inputs, int refresh_period, float ymin, float ymax, float xmin, float xmax, float label_max){
    blc_channel *input=nullptr;
    char const *gnuplot_format=NULL;
    char command[BLC_LINE_MAX];
    int i, offset=0, columns_nb=0, rows_nb=0;
    size_t element_size;
    int ret;
        
    input=&inputs.front();
    
    switch (input->dims_nb){
        case 0:
            columns_nb=1;
            rows_nb=1;
            break;
        case 1:
            columns_nb=inputs[0].dims[0].length;
            rows_nb=1;
            break;
        case 2:
            columns_nb=inputs[0].dims[0].length;
            rows_nb=inputs[0].dims[1].length;
            break;
        default:
            EXIT_ON_ARRAY_ERROR(&inputs[0], "Too many dims");
            break;
    }
    element_size=input->get_type_size();
    switch (input->type){
        case 'UIN8':gnuplot_format="%uchar";break;
        case 'INT8':gnuplot_format="%char";break;
        case 'IN16':gnuplot_format="%int16";break;
        case 'UI16':gnuplot_format="%uint16";break;
        case 'IN32':gnuplot_format="%int32";break;
        case 'UI32':gnuplot_format="%uint32";break;
        case 'FL32':gnuplot_format="%float32";break;
        case 'FL64':gnuplot_format="%float64";break;
        default: EXIT_ON_ARRAY_ERROR(input, "The type is not managed");
    }
    
    if (ymin!=ymax) fprintf(pipef, "set yrange [%f:%f]\n", ymin, ymax);
    if (columns_nb==1) xmin=-1;
    if (xmin!=xmax) fprintf(pipef, "set xrange [%f:%f]\n", xmin, xmax);
    
    if (with_option==nullptr){
        if  (inputs[0].total_length==1) with_option="boxes";
        else with_option="lines";
    }
    
    if (input->dims_nb==2 && input->dims[0].length>1 && input->dims[1].length>1){
        if (inputs.size()>1) EXIT_ON_ERROR("Multiple array of dims > 1 are not managed");
        if (surface_option)
        {
            fprintf(pipef, "set yrange [%f:%f]\n", 0.f, static_cast<double>(input->dims[1].length));
            fprintf(pipef, "set zrange [%f:%f]\n", ymin, ymax);
            fprintf(pipef, "set view 30,190\n");
            offset=snprintf(command, BLC_LINE_MAX, "splot '-' binary format='%s' array=%dx%d  title 'values' with pm3d ", gnuplot_format, columns_nb, rows_nb);
        }
        else{
            if (input->dims[1].length>64) EXIT_ON_ERROR("To many ( '%d' > 64 ) curves to draw", input->dims[1].length );
            offset=snprintf(command, BLC_LINE_MAX, "plot '-' binary format='%s' record=%zu using ($0*%f):1 title '0' with %s", gnuplot_format, input->dims[0].length, label_max/(float)input->dims[0].length, with_option);
            for(i=1; i<input->dims[1].length;i++){
                offset+=snprintf(command+offset, BLC_LINE_MAX-offset, ", '-'  binary format='%s' record=%zu   using ($0*%f):1 title '%d' with %s", gnuplot_format, input->dims[0].length,  label_max/(float)input->dims[0].length, i, with_option);
            }
            fprintf(stderr, "%s", command);
        }
    }
    else{
        offset=snprintf(command, BLC_LINE_MAX, "plot '-' binary format='%s' record=%zu using ($0*%f):1 title '%s' with %s", gnuplot_format, inputs[0].total_length, label_max/(float)inputs[0].total_length, input->name+1, with_option);
        for(i=1; i<inputs.size();i++){
            offset+=snprintf(command+offset, BLC_LINE_MAX-offset, ", '-'  binary format='%s' record=%zu  using ($0*%f):1 title '%s' with %s", gnuplot_format, inputs[i].total_length, label_max/(float)inputs[i].total_length, inputs[i].name+1, with_option);
        }
    }
    
    // sleep(3);
    for (i=0;i<inputs.size();i++) {
        blc_loop_try_add_waiting_semaphore(inputs[i].sem_new_data);
        blc_loop_try_add_posting_semaphore(inputs[i].sem_ack_data);
    }
    
    blc_command_add("c", [](char *argument, void *pipef ) -> void {
        fprintf((FILE*)pipef, "\n%s\n", argument);
       }, "string", "send command to gnuplot", pipef);
    blc_command_add("x", [](char *argument, void *pipef ) -> void {
           fprintf((FILE*)pipef, "\nset xrange [%s:]\n", argument);
          }, "real", "min x value", pipef);
       blc_command_add("X", [](char *argument, void *pipef ) -> void {
           fprintf((FILE*)pipef, "\nset xrange [:%s]\n", argument);
          }, "real", "max x value", pipef);
    blc_command_add("m", [](char *argument, void *pipef ) -> void {
        fprintf((FILE*)pipef, "\nset yrange [%s:]\n", argument);
       }, "real", "min y value", pipef);
    blc_command_add("M", [](char *argument, void *pipef ) -> void {
        fprintf((FILE*)pipef, "\nset yrange [:%s]\n", argument);
       }, "real", "max y value", pipef);

    
    BLC_COMMAND_LOOP(refresh_period){
        
        fprintf(pipef, "\n%s\n", command);
        for (i=0;i<inputs.size();i++) {
            SYSTEM_SUCCESS_CHECK(fwrite(inputs[i].data, element_size, columns_nb*rows_nb, pipef), columns_nb*rows_nb, NULL);
        }
        SYSTEM_SUCCESS_CHECK(fflush(pipef), 0, nullptr);
    }
    SYSTEM_ERROR_CHECK(ret=pclose(pipef), -1, nullptr);
    if (ret!=0) EXIT_ON_ERROR("Gnuplot quititing with error code '%d'", ret);
}



